The janux model defines the notion of a PermissionContext, which represents a business context and the permissions that may be defined in that context.  At its most basic level, this is simply a string name for the context, and a sorted list of strings identifying the individual permissions.  

(The sorting is not strictly necessary from a business standpoint, but we will see further below that defining it as a sorted list makes it possible to use bit logic to determine whether a given PermissionHolder has been granted a certain set of permissions.)

For example, in the classic case of a business application that needs to persist (CRUD) data on an entity, for example a Real Estate Property, a permission context may be defined as the tuple:

"PROPERTY_INFO":  ["READ", "UPDATE", "CREATE", "DELETE"]

An even more basic example would be the case of a monitoring application that needs to manage pre-defined Alert records: 

"ALERT": ["VIEW", "MODIFY"]

A commont anti-pattern is also to define the permissions according to the UI, to hide/disply certain parts of the UI:

"NAVBAR": ["PRODUCT", "SERVICES", "PROMOS", "USERS", ...]

While this there is nothing us stopping us from doing this, this is an anti-pattern because defining permissions too closely to the UI can lead to more rework when the UI changes.  Nevertheless, there are some cases, for example when provisioning 'Commercial Modules' to an application, where it may be very appropriate to define Permission Contexts around each Module.

In general terms, in the general case where your application Creates/Reads/Updates/Deletes (CRUD) a set of entities defined by an object model, we recommend that you start by defining the permissions by entity, as such:

"PRODUCT": ["READ","UPDATE","CREATE","TRASH","PURGE"]

or in a simple case where there is no notion of creation/deletion, for example in the case of a pre-defined configuration object that always exists and cannot be deleted, but can be changed:

"ALERT": ["READ","UPDATE"]


We don't recommend the term 'delete' because it is somewhat ambiguous.  Instead, we prefer the terms 'TRASH' and 'PURGE'. 

TRASH state represents the state of an entity that has been marked as inactive/hidden/in-trash, but has not been completely purged from the system.  This is often the case with data in relational databases, where one cannot delete a product, for example, which may still be referenced by past orders that included that product.  In such circumstances, it is often desirable to mark such record as inactive/hidden/in-trash, in order to conceal it from a UI, or prevent it from returning in current record queries, etc...  

In such cases, fully purging the record (deleting records marked inactive, or emptying the trash) would involve a potentially more disruptive administrative operations where it may be necessary to archive the purged records, or copy them to current records that reference them (such as an order history).  The "PURGE" permission represents this permission, which is usually only granted to data admistrator roles, rather than business users.

We also recommend that the order of the permissions be consistent from one entity to another, as this will make the job of troubleshooting permissions much easier.  We used the order above rather than the standard 'CRUD' order, because it makes it possible to define a simple Read/Update permission, and keep the 'READ', 'UPDATE' permission bits in the same positions (read more on the implementation details to understand why this is useful).

Of course, what names you want to use and what order you want to define them is completely up to you, as should be specific to the domain of your application.  For example, it may be more suitable to use the terms 'VIEW', 'MODIFY' rather than 'READ','UPDATE' in some contexts.  To this end, the implementation allows for display labels that can be changed and shown to a business user.  

Note, nevertheless, that the string that identifies the permission, or the permission context, cannot usually be changed easily, as it will require that the name be changed wherever it is being used, whether in a UI page or authorization configuration file.  Therefore, it is best to have general guidelines for how permissions should be named and defined, again to reduce the cost of troubleshooting, maintaining and evolving the Permission or Authorization Scheme.
