/** janux-security.js | Copyright (c) 2005-2013 www.janux.org | www.apache.org/licenses/LICENSE-2.0.html */
(function (root, factory) {
 	if (typeof define === 'function' && define.amd) {
		// use this lib as an AMD module; the other project will only see this AMD call, not the
		// internal modules in the closure below
		define(factory);
	} else {
		root.jnx = root.jnx || {};
		root.jnx.sec = factory();
	}
 } (this, function() {

/**
 * almond 0.2.5 Copyright (c) 2011-2012, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/jrburke/almond for details
 */
//Going sloppy to avoid 'use strict' string cost, but strict practices should
//be followed.
/*jslint sloppy: true */
/*global setTimeout: false */

var requirejs, require, define;
(function (undef) {
    var main, req, makeMap, handlers,
        defined = {},
        waiting = {},
        config = {},
        defining = {},
        hasOwn = Object.prototype.hasOwnProperty,
        aps = [].slice;

    function hasProp(obj, prop) {
        return hasOwn.call(obj, prop);
    }

    /**
     * Given a relative module name, like ./something, normalize it to
     * a real name that can be mapped to a path.
     * @param {String} name the relative name
     * @param {String} baseName a real name that the name arg is relative
     * to.
     * @returns {String} normalized name
     */
    function normalize(name, baseName) {
        var nameParts, nameSegment, mapValue, foundMap,
            foundI, foundStarMap, starI, i, j, part,
            baseParts = baseName && baseName.split("/"),
            map = config.map,
            starMap = (map && map['*']) || {};

        //Adjust any relative paths.
        if (name && name.charAt(0) === ".") {
            //If have a base name, try to normalize against it,
            //otherwise, assume it is a top-level require that will
            //be relative to baseUrl in the end.
            if (baseName) {
                //Convert baseName to array, and lop off the last part,
                //so that . matches that "directory" and not name of the baseName's
                //module. For instance, baseName of "one/two/three", maps to
                //"one/two/three.js", but we want the directory, "one/two" for
                //this normalization.
                baseParts = baseParts.slice(0, baseParts.length - 1);

                name = baseParts.concat(name.split("/"));

                //start trimDots
                for (i = 0; i < name.length; i += 1) {
                    part = name[i];
                    if (part === ".") {
                        name.splice(i, 1);
                        i -= 1;
                    } else if (part === "..") {
                        if (i === 1 && (name[2] === '..' || name[0] === '..')) {
                            //End of the line. Keep at least one non-dot
                            //path segment at the front so it can be mapped
                            //correctly to disk. Otherwise, there is likely
                            //no path mapping for a path starting with '..'.
                            //This can still fail, but catches the most reasonable
                            //uses of ..
                            break;
                        } else if (i > 0) {
                            name.splice(i - 1, 2);
                            i -= 2;
                        }
                    }
                }
                //end trimDots

                name = name.join("/");
            } else if (name.indexOf('./') === 0) {
                // No baseName, so this is ID is resolved relative
                // to baseUrl, pull off the leading dot.
                name = name.substring(2);
            }
        }

        //Apply map config if available.
        if ((baseParts || starMap) && map) {
            nameParts = name.split('/');

            for (i = nameParts.length; i > 0; i -= 1) {
                nameSegment = nameParts.slice(0, i).join("/");

                if (baseParts) {
                    //Find the longest baseName segment match in the config.
                    //So, do joins on the biggest to smallest lengths of baseParts.
                    for (j = baseParts.length; j > 0; j -= 1) {
                        mapValue = map[baseParts.slice(0, j).join('/')];

                        //baseName segment has  config, find if it has one for
                        //this name.
                        if (mapValue) {
                            mapValue = mapValue[nameSegment];
                            if (mapValue) {
                                //Match, update name to the new value.
                                foundMap = mapValue;
                                foundI = i;
                                break;
                            }
                        }
                    }
                }

                if (foundMap) {
                    break;
                }

                //Check for a star map match, but just hold on to it,
                //if there is a shorter segment match later in a matching
                //config, then favor over this star map.
                if (!foundStarMap && starMap && starMap[nameSegment]) {
                    foundStarMap = starMap[nameSegment];
                    starI = i;
                }
            }

            if (!foundMap && foundStarMap) {
                foundMap = foundStarMap;
                foundI = starI;
            }

            if (foundMap) {
                nameParts.splice(0, foundI, foundMap);
                name = nameParts.join('/');
            }
        }

        return name;
    }

    function makeRequire(relName, forceSync) {
        return function () {
            //A version of a require function that passes a moduleName
            //value for items that may need to
            //look up paths relative to the moduleName
            return req.apply(undef, aps.call(arguments, 0).concat([relName, forceSync]));
        };
    }

    function makeNormalize(relName) {
        return function (name) {
            return normalize(name, relName);
        };
    }

    function makeLoad(depName) {
        return function (value) {
            defined[depName] = value;
        };
    }

    function callDep(name) {
        if (hasProp(waiting, name)) {
            var args = waiting[name];
            delete waiting[name];
            defining[name] = true;
            main.apply(undef, args);
        }

        if (!hasProp(defined, name) && !hasProp(defining, name)) {
            throw new Error('No ' + name);
        }
        return defined[name];
    }

    //Turns a plugin!resource to [plugin, resource]
    //with the plugin being undefined if the name
    //did not have a plugin prefix.
    function splitPrefix(name) {
        var prefix,
            index = name ? name.indexOf('!') : -1;
        if (index > -1) {
            prefix = name.substring(0, index);
            name = name.substring(index + 1, name.length);
        }
        return [prefix, name];
    }

    /**
     * Makes a name map, normalizing the name, and using a plugin
     * for normalization if necessary. Grabs a ref to plugin
     * too, as an optimization.
     */
    makeMap = function (name, relName) {
        var plugin,
            parts = splitPrefix(name),
            prefix = parts[0];

        name = parts[1];

        if (prefix) {
            prefix = normalize(prefix, relName);
            plugin = callDep(prefix);
        }

        //Normalize according
        if (prefix) {
            if (plugin && plugin.normalize) {
                name = plugin.normalize(name, makeNormalize(relName));
            } else {
                name = normalize(name, relName);
            }
        } else {
            name = normalize(name, relName);
            parts = splitPrefix(name);
            prefix = parts[0];
            name = parts[1];
            if (prefix) {
                plugin = callDep(prefix);
            }
        }

        //Using ridiculous property names for space reasons
        return {
            f: prefix ? prefix + '!' + name : name, //fullName
            n: name,
            pr: prefix,
            p: plugin
        };
    };

    function makeConfig(name) {
        return function () {
            return (config && config.config && config.config[name]) || {};
        };
    }

    handlers = {
        require: function (name) {
            return makeRequire(name);
        },
        exports: function (name) {
            var e = defined[name];
            if (typeof e !== 'undefined') {
                return e;
            } else {
                return (defined[name] = {});
            }
        },
        module: function (name) {
            return {
                id: name,
                uri: '',
                exports: defined[name],
                config: makeConfig(name)
            };
        }
    };

    main = function (name, deps, callback, relName) {
        var cjsModule, depName, ret, map, i,
            args = [],
            usingExports;

        //Use name if no relName
        relName = relName || name;

        //Call the callback to define the module, if necessary.
        if (typeof callback === 'function') {

            //Pull out the defined dependencies and pass the ordered
            //values to the callback.
            //Default to [require, exports, module] if no deps
            deps = !deps.length && callback.length ? ['require', 'exports', 'module'] : deps;
            for (i = 0; i < deps.length; i += 1) {
                map = makeMap(deps[i], relName);
                depName = map.f;

                //Fast path CommonJS standard dependencies.
                if (depName === "require") {
                    args[i] = handlers.require(name);
                } else if (depName === "exports") {
                    //CommonJS module spec 1.1
                    args[i] = handlers.exports(name);
                    usingExports = true;
                } else if (depName === "module") {
                    //CommonJS module spec 1.1
                    cjsModule = args[i] = handlers.module(name);
                } else if (hasProp(defined, depName) ||
                           hasProp(waiting, depName) ||
                           hasProp(defining, depName)) {
                    args[i] = callDep(depName);
                } else if (map.p) {
                    map.p.load(map.n, makeRequire(relName, true), makeLoad(depName), {});
                    args[i] = defined[depName];
                } else {
                    throw new Error(name + ' missing ' + depName);
                }
            }

            ret = callback.apply(defined[name], args);

            if (name) {
                //If setting exports via "module" is in play,
                //favor that over return value and exports. After that,
                //favor a non-undefined return value over exports use.
                if (cjsModule && cjsModule.exports !== undef &&
                        cjsModule.exports !== defined[name]) {
                    defined[name] = cjsModule.exports;
                } else if (ret !== undef || !usingExports) {
                    //Use the return value from the function.
                    defined[name] = ret;
                }
            }
        } else if (name) {
            //May just be an object definition for the module. Only
            //worry about defining if have a module name.
            defined[name] = callback;
        }
    };

    requirejs = require = req = function (deps, callback, relName, forceSync, alt) {
        if (typeof deps === "string") {
            if (handlers[deps]) {
                //callback in this case is really relName
                return handlers[deps](callback);
            }
            //Just return the module wanted. In this scenario, the
            //deps arg is the module name, and second arg (if passed)
            //is just the relName.
            //Normalize module name, if it contains . or ..
            return callDep(makeMap(deps, callback).f);
        } else if (!deps.splice) {
            //deps is a config object, not an array.
            config = deps;
            if (callback.splice) {
                //callback is an array, which means it is a dependency list.
                //Adjust args if there are dependencies
                deps = callback;
                callback = relName;
                relName = null;
            } else {
                deps = undef;
            }
        }

        //Support require(['a'])
        callback = callback || function () {};

        //If relName is a function, it is an errback handler,
        //so remove it.
        if (typeof relName === 'function') {
            relName = forceSync;
            forceSync = alt;
        }

        //Simulate async callback;
        if (forceSync) {
            main(undef, deps, callback, relName);
        } else {
            //Using a non-zero value because of concern for what old browsers
            //do, and latest browsers "upgrade" to 4 if lower value is used:
            //http://www.whatwg.org/specs/web-apps/current-work/multipage/timers.html#dom-windowtimers-settimeout:
            //If want a value immediately, use require('id') instead -- something
            //that works in almond on the global level, but not guaranteed and
            //unlikely to work in other AMD implementations.
            setTimeout(function () {
                main(undef, deps, callback, relName);
            }, 4);
        }

        return req;
    };

    /**
     * Just drops the config on the floor, but returns req in case
     * the config return value is used.
     */
    req.config = function (cfg) {
        config = cfg;
        if (config.deps) {
            req(config.deps, config.callback);
        }
        return req;
    };

    define = function (name, deps, callback) {

        //This module may not have dependencies
        if (!deps.splice) {
            //deps is not an array, so probably means
            //an object literal or factory function for
            //the value. Adjust args.
            callback = deps;
            deps = [];
        }

        if (!hasProp(defined, name) && !hasProp(waiting, name)) {
            waiting[name] = [name, deps, callback];
        }
    };

    define.amd = {
        jQuery: true
    };
}());

define("almond", function(){});

(function(){var n=this,t=n._,r={},e=Array.prototype,u=Object.prototype,i=Function.prototype,a=e.push,o=e.slice,c=e.concat,l=u.toString,f=u.hasOwnProperty,s=e.forEach,p=e.map,h=e.reduce,v=e.reduceRight,d=e.filter,g=e.every,m=e.some,y=e.indexOf,b=e.lastIndexOf,x=Array.isArray,_=Object.keys,j=i.bind,w=function(n){return n instanceof w?n:this instanceof w?(this._wrapped=n,void 0):new w(n)};"undefined"!=typeof exports?("undefined"!=typeof module&&module.exports&&(exports=module.exports=w),exports._=w):n._=w,w.VERSION="1.4.4";var A=w.each=w.forEach=function(n,t,e){if(null!=n)if(s&&n.forEach===s)n.forEach(t,e);else if(n.length===+n.length){for(var u=0,i=n.length;i>u;u++)if(t.call(e,n[u],u,n)===r)return}else for(var a in n)if(w.has(n,a)&&t.call(e,n[a],a,n)===r)return};w.map=w.collect=function(n,t,r){var e=[];return null==n?e:p&&n.map===p?n.map(t,r):(A(n,function(n,u,i){e[e.length]=t.call(r,n,u,i)}),e)};var O="Reduce of empty array with no initial value";w.reduce=w.foldl=w.inject=function(n,t,r,e){var u=arguments.length>2;if(null==n&&(n=[]),h&&n.reduce===h)return e&&(t=w.bind(t,e)),u?n.reduce(t,r):n.reduce(t);if(A(n,function(n,i,a){u?r=t.call(e,r,n,i,a):(r=n,u=!0)}),!u)throw new TypeError(O);return r},w.reduceRight=w.foldr=function(n,t,r,e){var u=arguments.length>2;if(null==n&&(n=[]),v&&n.reduceRight===v)return e&&(t=w.bind(t,e)),u?n.reduceRight(t,r):n.reduceRight(t);var i=n.length;if(i!==+i){var a=w.keys(n);i=a.length}if(A(n,function(o,c,l){c=a?a[--i]:--i,u?r=t.call(e,r,n[c],c,l):(r=n[c],u=!0)}),!u)throw new TypeError(O);return r},w.find=w.detect=function(n,t,r){var e;return E(n,function(n,u,i){return t.call(r,n,u,i)?(e=n,!0):void 0}),e},w.filter=w.select=function(n,t,r){var e=[];return null==n?e:d&&n.filter===d?n.filter(t,r):(A(n,function(n,u,i){t.call(r,n,u,i)&&(e[e.length]=n)}),e)},w.reject=function(n,t,r){return w.filter(n,function(n,e,u){return!t.call(r,n,e,u)},r)},w.every=w.all=function(n,t,e){t||(t=w.identity);var u=!0;return null==n?u:g&&n.every===g?n.every(t,e):(A(n,function(n,i,a){return(u=u&&t.call(e,n,i,a))?void 0:r}),!!u)};var E=w.some=w.any=function(n,t,e){t||(t=w.identity);var u=!1;return null==n?u:m&&n.some===m?n.some(t,e):(A(n,function(n,i,a){return u||(u=t.call(e,n,i,a))?r:void 0}),!!u)};w.contains=w.include=function(n,t){return null==n?!1:y&&n.indexOf===y?n.indexOf(t)!=-1:E(n,function(n){return n===t})},w.invoke=function(n,t){var r=o.call(arguments,2),e=w.isFunction(t);return w.map(n,function(n){return(e?t:n[t]).apply(n,r)})},w.pluck=function(n,t){return w.map(n,function(n){return n[t]})},w.where=function(n,t,r){return w.isEmpty(t)?r?null:[]:w[r?"find":"filter"](n,function(n){for(var r in t)if(t[r]!==n[r])return!1;return!0})},w.findWhere=function(n,t){return w.where(n,t,!0)},w.max=function(n,t,r){if(!t&&w.isArray(n)&&n[0]===+n[0]&&65535>n.length)return Math.max.apply(Math,n);if(!t&&w.isEmpty(n))return-1/0;var e={computed:-1/0,value:-1/0};return A(n,function(n,u,i){var a=t?t.call(r,n,u,i):n;a>=e.computed&&(e={value:n,computed:a})}),e.value},w.min=function(n,t,r){if(!t&&w.isArray(n)&&n[0]===+n[0]&&65535>n.length)return Math.min.apply(Math,n);if(!t&&w.isEmpty(n))return 1/0;var e={computed:1/0,value:1/0};return A(n,function(n,u,i){var a=t?t.call(r,n,u,i):n;e.computed>a&&(e={value:n,computed:a})}),e.value},w.shuffle=function(n){var t,r=0,e=[];return A(n,function(n){t=w.random(r++),e[r-1]=e[t],e[t]=n}),e};var k=function(n){return w.isFunction(n)?n:function(t){return t[n]}};w.sortBy=function(n,t,r){var e=k(t);return w.pluck(w.map(n,function(n,t,u){return{value:n,index:t,criteria:e.call(r,n,t,u)}}).sort(function(n,t){var r=n.criteria,e=t.criteria;if(r!==e){if(r>e||r===void 0)return 1;if(e>r||e===void 0)return-1}return n.index<t.index?-1:1}),"value")};var F=function(n,t,r,e){var u={},i=k(t||w.identity);return A(n,function(t,a){var o=i.call(r,t,a,n);e(u,o,t)}),u};w.groupBy=function(n,t,r){return F(n,t,r,function(n,t,r){(w.has(n,t)?n[t]:n[t]=[]).push(r)})},w.countBy=function(n,t,r){return F(n,t,r,function(n,t){w.has(n,t)||(n[t]=0),n[t]++})},w.sortedIndex=function(n,t,r,e){r=null==r?w.identity:k(r);for(var u=r.call(e,t),i=0,a=n.length;a>i;){var o=i+a>>>1;u>r.call(e,n[o])?i=o+1:a=o}return i},w.toArray=function(n){return n?w.isArray(n)?o.call(n):n.length===+n.length?w.map(n,w.identity):w.values(n):[]},w.size=function(n){return null==n?0:n.length===+n.length?n.length:w.keys(n).length},w.first=w.head=w.take=function(n,t,r){return null==n?void 0:null==t||r?n[0]:o.call(n,0,t)},w.initial=function(n,t,r){return o.call(n,0,n.length-(null==t||r?1:t))},w.last=function(n,t,r){return null==n?void 0:null==t||r?n[n.length-1]:o.call(n,Math.max(n.length-t,0))},w.rest=w.tail=w.drop=function(n,t,r){return o.call(n,null==t||r?1:t)},w.compact=function(n){return w.filter(n,w.identity)};var R=function(n,t,r){return A(n,function(n){w.isArray(n)?t?a.apply(r,n):R(n,t,r):r.push(n)}),r};w.flatten=function(n,t){return R(n,t,[])},w.without=function(n){return w.difference(n,o.call(arguments,1))},w.uniq=w.unique=function(n,t,r,e){w.isFunction(t)&&(e=r,r=t,t=!1);var u=r?w.map(n,r,e):n,i=[],a=[];return A(u,function(r,e){(t?e&&a[a.length-1]===r:w.contains(a,r))||(a.push(r),i.push(n[e]))}),i},w.union=function(){return w.uniq(c.apply(e,arguments))},w.intersection=function(n){var t=o.call(arguments,1);return w.filter(w.uniq(n),function(n){return w.every(t,function(t){return w.indexOf(t,n)>=0})})},w.difference=function(n){var t=c.apply(e,o.call(arguments,1));return w.filter(n,function(n){return!w.contains(t,n)})},w.zip=function(){for(var n=o.call(arguments),t=w.max(w.pluck(n,"length")),r=Array(t),e=0;t>e;e++)r[e]=w.pluck(n,""+e);return r},w.object=function(n,t){if(null==n)return{};for(var r={},e=0,u=n.length;u>e;e++)t?r[n[e]]=t[e]:r[n[e][0]]=n[e][1];return r},w.indexOf=function(n,t,r){if(null==n)return-1;var e=0,u=n.length;if(r){if("number"!=typeof r)return e=w.sortedIndex(n,t),n[e]===t?e:-1;e=0>r?Math.max(0,u+r):r}if(y&&n.indexOf===y)return n.indexOf(t,r);for(;u>e;e++)if(n[e]===t)return e;return-1},w.lastIndexOf=function(n,t,r){if(null==n)return-1;var e=null!=r;if(b&&n.lastIndexOf===b)return e?n.lastIndexOf(t,r):n.lastIndexOf(t);for(var u=e?r:n.length;u--;)if(n[u]===t)return u;return-1},w.range=function(n,t,r){1>=arguments.length&&(t=n||0,n=0),r=arguments[2]||1;for(var e=Math.max(Math.ceil((t-n)/r),0),u=0,i=Array(e);e>u;)i[u++]=n,n+=r;return i},w.bind=function(n,t){if(n.bind===j&&j)return j.apply(n,o.call(arguments,1));var r=o.call(arguments,2);return function(){return n.apply(t,r.concat(o.call(arguments)))}},w.partial=function(n){var t=o.call(arguments,1);return function(){return n.apply(this,t.concat(o.call(arguments)))}},w.bindAll=function(n){var t=o.call(arguments,1);return 0===t.length&&(t=w.functions(n)),A(t,function(t){n[t]=w.bind(n[t],n)}),n},w.memoize=function(n,t){var r={};return t||(t=w.identity),function(){var e=t.apply(this,arguments);return w.has(r,e)?r[e]:r[e]=n.apply(this,arguments)}},w.delay=function(n,t){var r=o.call(arguments,2);return setTimeout(function(){return n.apply(null,r)},t)},w.defer=function(n){return w.delay.apply(w,[n,1].concat(o.call(arguments,1)))},w.throttle=function(n,t){var r,e,u,i,a=0,o=function(){a=new Date,u=null,i=n.apply(r,e)};return function(){var c=new Date,l=t-(c-a);return r=this,e=arguments,0>=l?(clearTimeout(u),u=null,a=c,i=n.apply(r,e)):u||(u=setTimeout(o,l)),i}},w.debounce=function(n,t,r){var e,u;return function(){var i=this,a=arguments,o=function(){e=null,r||(u=n.apply(i,a))},c=r&&!e;return clearTimeout(e),e=setTimeout(o,t),c&&(u=n.apply(i,a)),u}},w.once=function(n){var t,r=!1;return function(){return r?t:(r=!0,t=n.apply(this,arguments),n=null,t)}},w.wrap=function(n,t){return function(){var r=[n];return a.apply(r,arguments),t.apply(this,r)}},w.compose=function(){var n=arguments;return function(){for(var t=arguments,r=n.length-1;r>=0;r--)t=[n[r].apply(this,t)];return t[0]}},w.after=function(n,t){return 0>=n?t():function(){return 1>--n?t.apply(this,arguments):void 0}},w.keys=_||function(n){if(n!==Object(n))throw new TypeError("Invalid object");var t=[];for(var r in n)w.has(n,r)&&(t[t.length]=r);return t},w.values=function(n){var t=[];for(var r in n)w.has(n,r)&&t.push(n[r]);return t},w.pairs=function(n){var t=[];for(var r in n)w.has(n,r)&&t.push([r,n[r]]);return t},w.invert=function(n){var t={};for(var r in n)w.has(n,r)&&(t[n[r]]=r);return t},w.functions=w.methods=function(n){var t=[];for(var r in n)w.isFunction(n[r])&&t.push(r);return t.sort()},w.extend=function(n){return A(o.call(arguments,1),function(t){if(t)for(var r in t)n[r]=t[r]}),n},w.pick=function(n){var t={},r=c.apply(e,o.call(arguments,1));return A(r,function(r){r in n&&(t[r]=n[r])}),t},w.omit=function(n){var t={},r=c.apply(e,o.call(arguments,1));for(var u in n)w.contains(r,u)||(t[u]=n[u]);return t},w.defaults=function(n){return A(o.call(arguments,1),function(t){if(t)for(var r in t)null==n[r]&&(n[r]=t[r])}),n},w.clone=function(n){return w.isObject(n)?w.isArray(n)?n.slice():w.extend({},n):n},w.tap=function(n,t){return t(n),n};var I=function(n,t,r,e){if(n===t)return 0!==n||1/n==1/t;if(null==n||null==t)return n===t;n instanceof w&&(n=n._wrapped),t instanceof w&&(t=t._wrapped);var u=l.call(n);if(u!=l.call(t))return!1;switch(u){case"[object String]":return n==t+"";case"[object Number]":return n!=+n?t!=+t:0==n?1/n==1/t:n==+t;case"[object Date]":case"[object Boolean]":return+n==+t;case"[object RegExp]":return n.source==t.source&&n.global==t.global&&n.multiline==t.multiline&&n.ignoreCase==t.ignoreCase}if("object"!=typeof n||"object"!=typeof t)return!1;for(var i=r.length;i--;)if(r[i]==n)return e[i]==t;r.push(n),e.push(t);var a=0,o=!0;if("[object Array]"==u){if(a=n.length,o=a==t.length)for(;a--&&(o=I(n[a],t[a],r,e)););}else{var c=n.constructor,f=t.constructor;if(c!==f&&!(w.isFunction(c)&&c instanceof c&&w.isFunction(f)&&f instanceof f))return!1;for(var s in n)if(w.has(n,s)&&(a++,!(o=w.has(t,s)&&I(n[s],t[s],r,e))))break;if(o){for(s in t)if(w.has(t,s)&&!a--)break;o=!a}}return r.pop(),e.pop(),o};w.isEqual=function(n,t){return I(n,t,[],[])},w.isEmpty=function(n){if(null==n)return!0;if(w.isArray(n)||w.isString(n))return 0===n.length;for(var t in n)if(w.has(n,t))return!1;return!0},w.isElement=function(n){return!(!n||1!==n.nodeType)},w.isArray=x||function(n){return"[object Array]"==l.call(n)},w.isObject=function(n){return n===Object(n)},A(["Arguments","Function","String","Number","Date","RegExp"],function(n){w["is"+n]=function(t){return l.call(t)=="[object "+n+"]"}}),w.isArguments(arguments)||(w.isArguments=function(n){return!(!n||!w.has(n,"callee"))}),"function"!=typeof/./&&(w.isFunction=function(n){return"function"==typeof n}),w.isFinite=function(n){return isFinite(n)&&!isNaN(parseFloat(n))},w.isNaN=function(n){return w.isNumber(n)&&n!=+n},w.isBoolean=function(n){return n===!0||n===!1||"[object Boolean]"==l.call(n)},w.isNull=function(n){return null===n},w.isUndefined=function(n){return n===void 0},w.has=function(n,t){return f.call(n,t)},w.noConflict=function(){return n._=t,this},w.identity=function(n){return n},w.times=function(n,t,r){for(var e=Array(n),u=0;n>u;u++)e[u]=t.call(r,u);return e},w.random=function(n,t){return null==t&&(t=n,n=0),n+Math.floor(Math.random()*(t-n+1))};var M={escape:{"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#x27;","/":"&#x2F;"}};M.unescape=w.invert(M.escape);var S={escape:RegExp("["+w.keys(M.escape).join("")+"]","g"),unescape:RegExp("("+w.keys(M.unescape).join("|")+")","g")};w.each(["escape","unescape"],function(n){w[n]=function(t){return null==t?"":(""+t).replace(S[n],function(t){return M[n][t]})}}),w.result=function(n,t){if(null==n)return null;var r=n[t];return w.isFunction(r)?r.call(n):r},w.mixin=function(n){A(w.functions(n),function(t){var r=w[t]=n[t];w.prototype[t]=function(){var n=[this._wrapped];return a.apply(n,arguments),D.call(this,r.apply(w,n))}})};var N=0;w.uniqueId=function(n){var t=++N+"";return n?n+t:t},w.templateSettings={evaluate:/<%([\s\S]+?)%>/g,interpolate:/<%=([\s\S]+?)%>/g,escape:/<%-([\s\S]+?)%>/g};var T=/(.)^/,q={"'":"'","\\":"\\","\r":"r","\n":"n","	":"t","\u2028":"u2028","\u2029":"u2029"},B=/\\|'|\r|\n|\t|\u2028|\u2029/g;w.template=function(n,t,r){var e;r=w.defaults({},r,w.templateSettings);var u=RegExp([(r.escape||T).source,(r.interpolate||T).source,(r.evaluate||T).source].join("|")+"|$","g"),i=0,a="__p+='";n.replace(u,function(t,r,e,u,o){return a+=n.slice(i,o).replace(B,function(n){return"\\"+q[n]}),r&&(a+="'+\n((__t=("+r+"))==null?'':_.escape(__t))+\n'"),e&&(a+="'+\n((__t=("+e+"))==null?'':__t)+\n'"),u&&(a+="';\n"+u+"\n__p+='"),i=o+t.length,t}),a+="';\n",r.variable||(a="with(obj||{}){\n"+a+"}\n"),a="var __t,__p='',__j=Array.prototype.join,"+"print=function(){__p+=__j.call(arguments,'');};\n"+a+"return __p;\n";try{e=Function(r.variable||"obj","_",a)}catch(o){throw o.source=a,o}if(t)return e(t,w);var c=function(n){return e.call(this,n,w)};return c.source="function("+(r.variable||"obj")+"){\n"+a+"}",c},w.chain=function(n){return w(n).chain()};var D=function(n){return this._chain?w(n).chain():n};w.mixin(w),A(["pop","push","reverse","shift","sort","splice","unshift"],function(n){var t=e[n];w.prototype[n]=function(){var r=this._wrapped;return t.apply(r,arguments),"shift"!=n&&"splice"!=n||0!==r.length||delete r[0],D.call(this,r)}}),A(["concat","join","slice"],function(n){var t=e[n];w.prototype[n]=function(){return D.call(this,t.apply(this._wrapped,arguments))}}),w.extend(w.prototype,{chain:function(){return this._chain=!0,this},value:function(){return this._wrapped}})}).call(this);
define("underscore", (function (global) {
    return function () {
        var ret, fn;
        return ret || global._;
    };
}(this)));

/*! sprintf.js | Copyright (c) 2007-2013 Alexandru Marasteanu <hello at alexei dot ro> | 3 clause BSD license */(function(e){function r(e){return Object.prototype.toString.call(e).slice(8,-1).toLowerCase()}function i(e,t){for(var n=[];t>0;n[--t]=e);return n.join("")}var t=function(){return t.cache.hasOwnProperty(arguments[0])||(t.cache[arguments[0]]=t.parse(arguments[0])),t.format.call(null,t.cache[arguments[0]],arguments)};t.format=function(e,n){var s=1,o=e.length,u="",a,f=[],l,c,h,p,d,v;for(l=0;l<o;l++){u=r(e[l]);if(u==="string")f.push(e[l]);else if(u==="array"){h=e[l];if(h[2]){a=n[s];for(c=0;c<h[2].length;c++){if(!a.hasOwnProperty(h[2][c]))throw t('[sprintf] property "%s" does not exist',h[2][c]);a=a[h[2][c]]}}else h[1]?a=n[h[1]]:a=n[s++];if(/[^s]/.test(h[8])&&r(a)!="number")throw t("[sprintf] expecting number but found %s",r(a));switch(h[8]){case"b":a=a.toString(2);break;case"c":a=String.fromCharCode(a);break;case"d":a=parseInt(a,10);break;case"e":a=h[7]?a.toExponential(h[7]):a.toExponential();break;case"f":a=h[7]?parseFloat(a).toFixed(h[7]):parseFloat(a);break;case"o":a=a.toString(8);break;case"s":a=(a=String(a))&&h[7]?a.substring(0,h[7]):a;break;case"u":a>>>=0;break;case"x":a=a.toString(16);break;case"X":a=a.toString(16).toUpperCase()}a=/[def]/.test(h[8])&&h[3]&&a>=0?"+"+a:a,d=h[4]?h[4]=="0"?"0":h[4].charAt(1):" ",v=h[6]-String(a).length,p=h[6]?i(d,v):"",f.push(h[5]?a+p:p+a)}}return f.join("")},t.cache={},t.parse=function(e){var t=e,n=[],r=[],i=0;while(t){if((n=/^[^\x25]+/.exec(t))!==null)r.push(n[0]);else if((n=/^\x25{2}/.exec(t))!==null)r.push("%");else{if((n=/^\x25(?:([1-9]\d*)\$|\(([^\)]+)\))?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-fosuxX])/.exec(t))===null)throw"[sprintf] huh?";if(n[2]){i|=1;var s=[],o=n[2],u=[];if((u=/^([a-z_][a-z_\d]*)/i.exec(o))===null)throw"[sprintf] huh?";s.push(u[1]);while((o=o.substring(u[0].length))!=="")if((u=/^\.([a-z_][a-z_\d]*)/i.exec(o))!==null)s.push(u[1]);else{if((u=/^\[(\d+)\]/.exec(o))===null)throw"[sprintf] huh?";s.push(u[1])}n[2]=s}else i|=2;if(i===3)throw"[sprintf] mixing positional and named placeholders is not (yet) supported";r.push(n)}t=t.substring(n[0].length)}return r};var n=function(e,n,r){return r=n.slice(0),r.splice(0,0,e),t.apply(null,r)};e.sprintf=t,e.vsprintf=n})(typeof exports!="undefined"?exports:window);
define("sprintf-js", (function (global) {
    return function () {
        var ret, fn;
        return ret || global.sprintf;
    };
}(this)));



define('PermissionContext',['require','exports','module','underscore','sprintf-js'],function (require, exports) {
	

	var 
		_ = require('underscore')
		,sprintf = require('sprintf-js').sprintf
	;

	function PermissionContext(aName, aDescription) {

		if (!_.isString(aName) || _.isEmpty(aName)) {
			throw new Error("Attempting to instantiate a PermissionContext without a name");
		}

		// public members without accessors
		this.name = aName;
		this.description = aDescription;

		//// private object that stores the private state of an instance
		var priv = {};

		priv.bit         = {}; // stores permission bits indexed by name
		priv.permBitList = []; // stores permission bits ordered by bit position

		/**
		* Defines read-only accessor 'bit' to map of permission bits map
		* This makes it possible to access an individual permission bit like, for example,
		*
		*   somePermContext.bit.READ
		*
		* or better yet, if all permission contexts are stored in a single map indexed by
		* their name, one could access a bit like: permissionContext.PERSON.bit.READ
		*/
		priv.spec = {
			bit: {
				enumerable: true
				,get: function() { return priv.bit;}
				,set: function() {} // do nothing
			}
		}

		this.spec = function spec() { return priv.spec; }

		//// privileged methods
		/**
		* Adds a PermissionBit to this PermissionContext, makes sure that there are no two PermissionBits
		* with the same name and that the value of PermissionBit.position is sequential and without
		* gaps
		*/
		this.addPermissionBit = function addPermissionBit(name, description, sortOrder) {

			if (!_.isString(name) || _.isEmpty(name)) {
				throw new Error( sprintf("Attempting to add a PermissionBit without a name to PermissionContext '%s'", this.name));
			}

			if (priv.bit[name]) {
				throw new Error( sprintf("A permission bit with name: '%s' already exists in PermissionContext '%s'", permBit.name, this.name));
			}

			var permBit = {};

			// make 'name' and 'position' fields immutable
			Object.defineProperties(permBit, {
				name:      { enumerable: true, value: name }
				,position: { enumerable: true, value: priv.permBitList.length } 
			});

			permBit.label = permBit.name; // mutable label that can be used instead of name for display purposes
			permBit.description = (_.isString(description) && !_.isEmpty(description)) ? description : "";
			permBit.sortOrder   = _.isNumber(sortOrder) ? sortOrder : permBit.position;  // default display order to bit position

			priv.permBitList[permBit.position] = permBit;
			
			// Add the permissionBit as an immutable object of the PermissionContext
			Object.defineProperty(priv.bit, permBit.name, {enumerable: true,  value: permBit});
		};

		/**
		* returns a clone of the PermissionBits as a list; adds to the memory
		* footprint, so use sparingly; you can also iterate over the
		* PermissionContext.bit map (object), which is indexed by bit name, 
		* as in permContext.bit.READ.
		*/
		this.getPermissionBitsAsList = function getPermissionBitsAsList() {
			return _.map(priv.permBitList, function(bit) { return _.clone(bit);});
		};

	}


	Object.defineProperty(PermissionContext.prototype, "typeName", {
		enumerable: true,
		value: "janux.security.PermissionContext"
	});

	/**
	* Converts the permission passed to a number, by calculating 2 to the power of
	* the bit position; for example, if the "CREATE" bit position is '3',
	* getPermissionAsNumber("CREATE") will return '8' (2 to the power of 3), or in
	* base-2 notation: '1000'
	*/
	PermissionContext.prototype.getPermissionAsNumber = function getPermissionAsNumber(permBitName) {
		if (!_.isString(permBitName)) {
			throw new Error ( "Argument to getPermissionAsNumber must be a string");
		}

		//var bit = this.getPermissionBit(permBitName);
		var bit = this.bit[permBitName];

		if (!_.isObject(bit)) {
			throw new Error ( sprintf("Cannot convert permission '%s' to number: it does not exist in PermissionContext '%s'", permBitName, this.name) );
		}

		return Math.pow(2,bit.position);
	};

	/**
	* Convenience method that converts a list of permissions specified by name to a
	* numeric representation, by calling getPermissionAsNumber for each permission,
	* and adding all the numbers thus returned.  For example, assuming that
	* getPermissionAsNumber("READ") returns 1, ad
	* getPermissionAsNumber("UPDATE") returns 2 (or '10' in base-2), and
	* getPermissionAsNumber("CREATE") returns 4 (or '100' in base-2)
	* then getPermissionsAsNumber(["READ","UPDATE","CREATE"]) would return 7 or 111
	* in base-2 notation.
	*/
	PermissionContext.prototype.getPermissionsAsNumber = function getPermissionsAsNumber(perms) {
		if (!_.isArray(perms)) {
			throw new Error ("Argument to getPermissionsAsNumber must be an array of strings");
		}

		var self = this;

		function sumPerms(out, perm) {
			return out + self.getPermissionAsNumber(perm);
		}

		return _.reduce(perms, sumPerms, 0);
	};

	/**
	* Passing the 'doShortVersion' boolean flag will return a barebones representation of the
	* PermissionContext; this is useful when serializing to JSON PermissionHolder entities (Role,
	* Account) that need to know about the PermissionContext metadata, but where it's desirable to keep
	* those JSON strings from being overly verbose.  The default JSON representation will return something
	* like:
	*
	* {"name":"PERSON",
	*   "description":"Defines permissions available on a Person entity",
	*   "typeName":"janux.security.PermissionContext",
	*   "bit":{
	*     "READ":{"position":0,"label":"READ","description":"Grants permission to READ a PERSON","sortOrder":0},
	*     "UPDATE":{"position":1,"label":"UPDATE","description":"Grants permission to UPDATE a PERSON","sortOrder":99}
	*   }
	* }
	*
	* whereas the short representation of the same PermissionContext would return:
	*
	* {"name":"PERSON",
	*   "bit":{
	*     "READ":{"position":0},
	*     "UPDATE":{"position":1}
	*   }
	* }
	*
	*/
	PermissionContext.prototype.toJSON = function toJSON(doShortVersion) {
		var out = {};
		out.name = this.name;

		if (!doShortVersion) {
			out.description = this.description;
			out.typeName = this.typeName;
		}

		for (var key in this.bit) {
			out.bit = out.bit || {};
			var aBit = {}
			aBit.position = this.bit[key].position

			if (!doShortVersion) {
				aBit.label = this.bit[key].label;
				aBit.description = this.bit[key].description;
				aBit.sortOrder = this.bit[key].sortOrder;
			}
			out.bit[key] = aBit;
		}

		return out;
	};

	/** creates an instance of a PermissionContext, name is a required field */
	exports.createInstance = function createInstance(aName, aDescription) {
		var out = new PermissionContext(aName, aDescription);
		Object.defineProperties(out, out.spec());
		delete out.spec; // delete accessor to private spec, so it cannot be tampered
		return out;
	};

	/** deserializes a PermissionContext from its canonical toJSON representation */
	exports.fromJSON = function fromJSON(obj) {
		var out = exports.createInstance(obj.name, obj.description);
		var bitlist = _.pairs(obj.bit);
		_.each(bitlist, function(tuple) { out.addPermissionBit(tuple[0], tuple[1].description, tuple[1].sortOrder); });
		return out;
	}

}); // end define
;


define('PermissionHolder',['require','exports','module','underscore'],function (require, exports) {
	

	// if we are being called from a node environment use require, 
	// otherwise expect dependency to be in context passed (namely the window object in a browser)
	var 
		_ = require('underscore')
	;

	/**
	* meant to be an abstract object, that defines an interface, and provides
	* method implementations that can be copied to child object instances (objects
	* that need to implement the PermissionHolder interface)
	*/
	function PermissionHolder() {
	}

	Object.defineProperty(PermissionHolder.prototype, "typeName", {
		configurable: true, // allow implementing classes to change the typeName
		enumerable: true,
		value: "janux.security.PermissionHolder"
	});

	/**
	* Almighty PermissionHolders have all privileges, except those that are
	* explicitly denied; useful for specifying admin users that automatically get
	* assigned any new permissions created.
	*/
	Object.defineProperty(PermissionHolder.prototype, "isAlmighty", {
		enumerable: true
		,get: function() { return this._isAlmighty; }
		,set: function(bool) { this._isAlmighty = bool; }
	});

	/**
		* Given a PermissionContext and an array of strings representing multiple
		* permissions available in that context, this method grants those permissions
		* to this PermissionHolder entity. This method returns 'this' and may be chained.
		*
		* The permissions granted by this method replace any direct permissions that
		* the PermissionHolder may already have in this PermissionContext. 
		*
		* If you would like to remove all Permissions granted directly to this entity
		* within a Permission Context, pass an empty array.
		*/
	PermissionHolder.prototype.grantPermissions = function grantPermissions(perms, permContext) {
		if (!_.isObject(permContext) && permContext.typeName != "janux.security.PermissionContext") {
			throw new Error("You must pass a valid PermissionContext when granting permissions");
		}

		if (!_.isArray(perms) && !_.isNumber(perms)) {
			throw new Error("You must pass either a number or an array of string permissions when granting permissions");
		}

		var permsValue = _.isArray(perms) ? permContext.getPermissionsAsNumber(perms) : perms;
		this._permsGranted = this._permsGranted || {};

		if (permsValue > 0) {
			this._permsGranted[permContext.name] = {context: permContext, grant: permsValue};
		} else {
			this._permsGranted[permContext.name] = null;
		}
		return this;
	};

	/**
		* Boolean function called to determine whether this PermissionHolder holds a specific
		* permission within a PermissionContext, for example, assuming that a
		* PermissionContext 'PERSON' exists with permissions bits ["READ", "UPDATE"], 
		* calling hasPermission("READ", "PERSON") would return true if the
		* PermissionHolder has been granted the READ permission in this PermissionContext,
		* presumably, this would enable the PermissionHolder to view records associated with a
		* Person entity in an application
		*/
	PermissionHolder.prototype.hasPermission = function hasPermission(perm, context) {
		if (!_.isString(perm)) {
			throw new Error("hasPermission must be called with a string representing a permission name");
		}
		return this.hasPermissions([perm], context);
	};

	PermissionHolder.prototype.hasPermissions = function hasPermissions(perms, context) {
		if (!_.isArray(perms)) {
			throw new Error("hasPermissions must be called with an array of strings representing permission names");
		}
		// almighty users have all permissions for now (TODO: add 'deny' mechanism)
		if (this.isAlmighty) return true;

		var permsGranted = this._permsGranted[context];
		if (!_.isObject(permsGranted)) { return false; }

		var permContext = permsGranted.context;

		// console.log('permContext:', JSON.stringify(permContext));

		var requiredPerms = -1;
		try
		{ 
			requiredPerms = permContext.getPermissionsAsNumber(perms);
		}
		catch (e)
		{
			console.warn("WARNING: " + e );
			return false;
		}

		// console.log('requiredPerms:', requiredPerms);
		// console.log('permsGranted.grant:', permsGranted.grant);

		var match = permsGranted.grant & requiredPerms;
		// console.log("match is: ", match);

		return match == requiredPerms;
	};

	/**
	 * This is the preferred manner to check for permissions, as it is the most english-readable one,
	 * yielding a call such as:
	 *
	 * role.can("WRITE", DOCUMENT)
	 * or
	 * role.can(["UPDATE", "DELETE"], DOCUMENT)
	 *
	 * The first argument may be a string representing the permission, or an array of strings if
	 * checking for multiple permissions
	 */
	PermissionHolder.prototype.can = function can(perms, context) {
		if (_.isArray(perms)) {
			return this.hasPermissions(perms, context);
		} else if (_.isString(perms)) {
			return this.hasPermission(perms, context);
		} else {
			return false;
		}
	}


	PermissionHolder.prototype.toJSON = function toJSON() {
		var out = _.clone(this);
		delete out._permsGranted;
		delete out._isAlmighty;
		var perm;
		// outputs permissionsGranted separately from permissionsContexts to make json msg more readable
		// "permissions": {
		//   "PROPERTY":{"grant":3},
		//   "ACCOUNT":{"deny":7,}   // revokes inherited permissions, not yet implemented
		//   "EQUIPMENT":{"grant":3, "deny":4} // edge case, not yet implemented
		// }
		for (var key in this._permsGranted) {
			out.permissions = out.permissions || {};
			perm = this._permsGranted[key];
			out.permissions[key] = {};
			if (perm.grant) {out.permissions[key].grant = perm.grant;}
			if (perm.deny)  {out.permissions[key].deny  = perm.deny;}

			out.permissionContexts = out.permissionContexts || [];
			out.permissionContexts.push(perm.context.toJSON(true)); // true = doShortVersion
		}
		
		return out;
	};

	exports.createInstance = function createInstance() {
		var out = new PermissionHolder();
		// Object.defineProperties(out, out.spec());
		return out;
	};

}); // end define
;


define('Role',['require','exports','module','underscore','./PermissionHolder','./PermissionContext'],function (require, exports) {
	

	var
		_ = require('underscore')
		,PermissionHolder  = require('./PermissionHolder')
		,PermissionContext = require('./PermissionContext')
	;

	function Role(aName, aDescription) {
		
		if (!_.isString(aName) || _.isEmpty(aName)) {
			throw new Error("Attempting to instantiate a Role without a name");
		}

		// public members without accessors
		this.name        = aName;
		this.description = aDescription;
		this.sortOrder   = 0;
		this.isEnabled   = true;

		// private member
		this._permsGranted = {};
	}

	Role.prototype = PermissionHolder.createInstance();

	// static, immutable 
	Object.defineProperty(Role.prototype, "typeName", {
		enumerable: true,
		value: "janux.security.Role"
	});


	exports.createInstance = function createInstance(aName, aDescription) {
		var out = new Role(aName, aDescription);

		// Object.defineProperties(out, out.spec());
		return out;
	};

	/** static method that deserializes a Role from its canonical toJSON representation */
	exports.fromJSON = function fromJSON(obj) {
		var out = exports.createInstance(obj.name, obj.description);
		out.sortOrder = obj.sortOrder;
		_.each(obj.permissionContexts, function(permContext) {
			out.grantPermissions(obj.permissions[permContext.name].grant, PermissionContext.fromJSON(permContext));
		});
		if (obj.isAlmighty) out.isAlmighty = true;

		return out;
	}

});

define("main", ['PermissionContext', 'Role'], function(PermContext, Role) {
	return { 
		PermissionContext: PermContext
		,Role: Role
	};
});
	return require('main');
}));
