if (typeof define !== 'function') { var define = require('amdefine')(module); }

define(function (require, exports) {
	'use strict';

	var
		_ = require('underscore')
		,PermissionHolder  = require('./PermissionHolder')
		,PermissionContext = require('./PermissionContext')
	;

	function Role(aName, aDescription) {
		
		if (!_.isString(aName) || _.isEmpty(aName)) {
			throw new Error("Attempting to instantiate a Role without a name");
		}

		// public members without accessors
		this.name        = aName;
		this.description = aDescription;
		this.sortOrder   = 0;
		this.isEnabled   = true;

		// private member
		this._permsGranted = {};
	}

	Role.prototype = PermissionHolder.createInstance();

	// static, immutable 
	Object.defineProperty(Role.prototype, "typeName", {
		enumerable: true,
		value: "janux.security.Role"
	});


	exports.createInstance = function createInstance(aName, aDescription) {
		var out = new Role(aName, aDescription);

		// Object.defineProperties(out, out.spec());
		return out;
	};

	/** static method that deserializes a Role from its canonical toJSON representation */
	exports.fromJSON = function fromJSON(obj) {
		var out = exports.createInstance(obj.name, obj.description);
		out.sortOrder = obj.sortOrder;
		_.each(obj.permissionContexts, function(permContext) {
			out.grantPermissions(obj.permissions[permContext.name].grant, PermissionContext.fromJSON(permContext));
		});
		if (obj.isAlmighty) out.isAlmighty = true;

		return out;
	}

});
